! -*- F90 -*-
BEGIN_PROVIDER [ real*8, ylm, (0:lmax,(-lmax):lmax,0:lmax,0:lmax) ]
BEGIN_DOC
! table of the YLM
END_DOC
  implicit none
  real*8 :: norm_lm,flm,flmu,flmuv,mfact
  integer :: l,m,t,u,v,vv,bern,ix,iy,i

! we store the components of the real solid harmonics
! explicit formulas from the book of Helgaker, page 215,
! formulas 6.4.47 to 6.4.49
!
! the components are stored as l,m,ix,iy
!  with ix+iy+iz=l, thus we do not need to store the z-exponent
! ylm(2,0,0,0) is the coefficient for the z^2 component of ylm(2,0)
!
  do l=0,lmax
   do m=-lmax,lmax
    do ix=0,lmax
     do iy=0,lmax
      ylm(l,m,ix,iy)=0.D0
     end do
    end do
   end do
  end do
!
  do l=0,lmax
! positive m-values, including zero
   do m=-lmax,lmax
!    norm_lm=(0.5D0**dble(abs(m)))/dble(fact(l))*sqrt(dble(fact(l+m)*fact(l-m)) )

! calculate (l+|m|)!/l! = (l+1)*(l+2)...(l+|m|)
! calculate (l-|m|)!/l! = 1/[ (l-|m|+1)(l-|m|+2)...l ]
    mfact=1.d0
    do i=1,abs(m)
     mfact=mfact*dble(l+i)/dble(l-abs(m)+i)
    end do
    norm_lm=(0.5D0**dble(abs(m)))*sqrt(mfact)

    if (m.ne.0) norm_lm=norm_lm*sqrt(2.D0)
    do t=0,(l-abs(m))/2
     flm=0.25**dble(t)*bern(l,t)*bern(l-t,abs(m)+t)
     do u=0,t
      flmu=flm*bern(t,u)
      if (m.ge.0) then
       do v=0,m/2
        flmuv=flmu*bern(m,2*v)
        if (mod(t+v,2).eq.1) flmuv=-flmuv
        ix=2*t+m-2*(u+v)
        iy=2*(u+v)
        ylm(l,m,ix,iy)=ylm(l,m,ix,iy)+flmuv*norm_lm
       end do
      else
       do v=0,(-m-1)/2
! vv=2v
        vv=2*v+1
        flmuv=flmu*bern(-m,vv)
        if (mod(t+v,2).eq.1) flmuv=-flmuv
        ix=2*t-m-2*u-vv
        iy=2*u+vv
        ylm(l,m,ix,iy)=ylm(l,m,ix,iy)+flmuv*norm_lm
       end do
      end if
     end do
    end do
   end do
  end do

END_PROVIDER 
